package jp.terasoluna.fw.batch.executor;

import java.net.URL;

import jp.terasoluna.fw.batch.executor.SecurityManagerEx.ExitException;
import jp.terasoluna.fw.batch.executor.concurrent.BatchThreadPoolTaskExecutor;
import jp.terasoluna.fw.ex.unit.util.SystemEnvUtils;
import jp.terasoluna.fw.ex.unit.util.TerasolunaPropertyUtils;
import junit.framework.TestCase;

import org.apache.commons.logging.Log;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.config.ConfigurableListableBeanFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.AbstractApplicationContext;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;

public class AsyncBatchExecutorTest extends TestCase {

    final SecurityManager sm = System.getSecurityManager();

    public AsyncBatchExecutorTest() {
        super();
    }

    @Override
    protected void setUp() throws Exception {
        super.setUp();
        TerasolunaPropertyUtils.saveProperties();
        System.setSecurityManager(new SecurityManagerEx());
    }

    @Override
    protected void tearDown() throws Exception {
        TerasolunaPropertyUtils.restoreProperties();
        SystemEnvUtils.restoreEnv();
        System.setSecurityManager(sm);
        super.tearDown();
    }

    /**
     * maineXg01 yُnz
     * 
     * <pre>
     * O
     * Eadmin.dataSourcebean`t@C擾queryDaonull
     * ENfoo
     * mF
     * EIAL025007̃Oo͂邱
     * EIAL025006̃Ofooo͂邱
     * E^[R[h255
     * </pre>
     */
    public void testMain01() throws Exception {
        TerasolunaPropertyUtils.addProperty("beanDefinition.admin.dataSource",
                "AsyncBatchExecutorTest01.xml");
        // TerasolunaPropertyUtils.addProperty("executor.endMonitoringFile",
        // "src/test/resources/test_terminate");

        try {
            AsyncBatchExecutor.main(new String[] { "foo" });
            fail("O͔܂");
        } catch (ExitException e) {
            assertEquals(255, e.state);
        }
    }

    /**
     * maineXg02 yُnz
     * 
     * <pre>
     * O
     * Eadmin.dataSourcebean`t@C擾updateDaonull
     * ENfoo
     * mF
     * EIAL025008̃Oo͂邱
     * EIAL025006̃Ofooo͂邱
     * E^[R[h255
     * </pre>
     */
    public void testMain02() throws Exception {
        TerasolunaPropertyUtils.addProperty("beanDefinition.admin.dataSource",
                "AsyncBatchExecutorTest02.xml");
        // TerasolunaPropertyUtils.addProperty("executor.endMonitoringFile",
        // "src/test/resources/test_terminate");

        try {
            AsyncBatchExecutor.main(new String[] { "foo" });
            fail("O͔܂");
        } catch (ExitException e) {
            assertEquals(255, e.state);
        }
    }

    /**
     * maineXg03 yُnz
     * 
     * <pre>
     * O
     * Eadmin.dataSourcebean`t@C擾transactionManagernull
     * ENfoo
     * mF
     * EIAL025016̃Oo͂邱
     * EIAL025006̃Ofooo͂邱
     * E^[R[h255
     * </pre>
     */
    public void testMain03() throws Exception {
        TerasolunaPropertyUtils.addProperty("beanDefinition.admin.dataSource",
                "AsyncBatchExecutorTest03.xml");
        // TerasolunaPropertyUtils.addProperty("executor.endMonitoringFile",
        // "src/test/resources/test_terminate");

        try {
            AsyncBatchExecutor.main(new String[] { "foo" });
            fail("O͔܂");
        } catch (ExitException e) {
            assertEquals(255, e.state);
        }
    }

    /**
     * maineXg04 ynz
     * 
     * <pre>
     * O
     * EN
     * EϐJOB_APP_CDݒ
     * mF
     * EIAL025006̃OjoAppCdo͂Ȃ
     * </pre>
     */
    public void testMain04() throws Exception {
        TerasolunaPropertyUtils.addProperty("beanDefinition.admin.dataSource",
                "AsyncBatchExecutorTest01.xml");
        // TerasolunaPropertyUtils.addProperty("executor.endMonitoringFile",
        // "src/test/resources/test_terminate");
        SystemEnvUtils.removeEnv(AsyncBatchExecutor.ENV_JOB_APP_CD);
        try {
            AsyncBatchExecutor.main(new String[] {});
            fail("O͔܂");
        } catch (ExitException e) {
            assertEquals(255, e.state);
        }
    }

    /**
     * maineXg05 ynz
     * 
     * <pre>
     * O
     * EN
     * EϐJOB_APP_CDhogeݒ
     * mF
     * EIAL025006̃Ohogeo͂邱
     * </pre>
     */
    public void testMain05() throws Exception {
        TerasolunaPropertyUtils.addProperty("beanDefinition.admin.dataSource",
                "AsyncBatchExecutorTest01.xml");
        // TerasolunaPropertyUtils.addProperty("executor.endMonitoringFile",
        // "src/test/resources/test_terminate");
        SystemEnvUtils.setEnv(AsyncBatchExecutor.ENV_JOB_APP_CD, "hoge");
        try {
            AsyncBatchExecutor.main(new String[] {});
            fail("O͔܂");
        } catch (ExitException e) {
            assertEquals(255, e.state);
        }
    }

    /**
     * maineXg06 ynz
     * 
     * <pre>
     * O
     * Ewf[^x[Xُ펞̃gC񐔁x&quot;abc&quot;ݒ
     * mF
     * E^[R[h255
     * </pre>
     */
    public void testMain06() throws Exception {
        TerasolunaPropertyUtils
                .removeProperty("batchTaskExecutor.dbAbnormalRetryMax");
        TerasolunaPropertyUtils.addProperty(
                "batchTaskExecutor.dbAbnormalRetryMax", "abc");

        SystemEnvUtils.setEnv(AsyncBatchExecutor.ENV_JOB_APP_CD, "hoge");
        try {
            AsyncBatchExecutor.main(new String[] {});
            fail("O͔܂");
        } catch (ExitException e) {
            assertEquals(255, e.state);
        }
    }

    /**
     * maineXg07 ynz
     * 
     * <pre>
     * O
     * Ewf[^x[Xُ펞̃gCԊui~bjx&quot;abc&quot;ݒ
     * mF
     * E^[R[h255
     * </pre>
     */
    public void testMain07() throws Exception {
        TerasolunaPropertyUtils
                .removeProperty("batchTaskExecutor.dbAbnormalRetryInterval");
        TerasolunaPropertyUtils.addProperty(
                "batchTaskExecutor.dbAbnormalRetryInterval", "abc");

        SystemEnvUtils.setEnv(AsyncBatchExecutor.ENV_JOB_APP_CD, "hoge");
        try {
            AsyncBatchExecutor.main(new String[] {});
            fail("O͔܂");
        } catch (ExitException e) {
            assertEquals(255, e.state);
        }
    }

    /**
     * maineXg08 ynz
     * 
     * <pre>
     * O
     * Ewf[^x[Xُ펞̃gC񐔂ZbgO񂩂̔Ԋui~bjx&quot;abc&quot;ݒ
     * mF
     * E^[R[h255
     * </pre>
     */
    public void testMain08() throws Exception {
        TerasolunaPropertyUtils
                .removeProperty("batchTaskExecutor.dbAbnormalRetryReset");
        TerasolunaPropertyUtils.addProperty(
                "batchTaskExecutor.dbAbnormalRetryReset", "abc");

        SystemEnvUtils.setEnv(AsyncBatchExecutor.ENV_JOB_APP_CD, "hoge");
        try {
            AsyncBatchExecutor.main(new String[] {});
            fail("O͔܂");
        } catch (ExitException e) {
            assertEquals(255, e.state);
        }
    }

    /**
     * testCloseRootApplicationContext001
     * @throws Exception
     */
    public void testCloseRootApplicationContext001() throws Exception {
        ApplicationContext context = null;
        AsyncBatchExecutor.closeRootApplicationContext(context);
    }

    /**
     * testCloseRootApplicationContext002
     * @throws Exception
     */
    public void testCloseRootApplicationContext002() throws Exception {
        ApplicationContext context = new AbstractApplicationContext() {

            @Override
            protected void closeBeanFactory() {
            }

            @Override
            public ConfigurableListableBeanFactory getBeanFactory()
                                                                   throws IllegalStateException {
                return null;
            }

            @Override
            protected void refreshBeanFactory() throws BeansException,
                                               IllegalStateException {
            }
        };

        AsyncBatchExecutor.closeRootApplicationContext(context);
    }

    /**
     * testLogOutputTaskExecutor001
     * @throws Exception
     */
    public void testLogOutputTaskExecutor001() throws Exception {
        Log log = new Log() {
            public void debug(Object message) {
            }

            public void debug(Object message, Throwable t) {
            }

            public void error(Object message) {
            }

            public void error(Object message, Throwable t) {
            }

            public void fatal(Object message) {
            }

            public void fatal(Object message, Throwable t) {
            }

            public void info(Object message) {
            }

            public void info(Object message, Throwable t) {
            }

            public boolean isDebugEnabled() {
                return false;
            }

            public boolean isErrorEnabled() {
                return false;
            }

            public boolean isFatalEnabled() {
                return false;
            }

            public boolean isInfoEnabled() {
                return false;
            }

            public boolean isTraceEnabled() {
                return false;
            }

            public boolean isWarnEnabled() {
                return false;
            }

            public void trace(Object message) {
            }

            public void trace(Object message, Throwable t) {
            }

            public void warn(Object message) {
            }

            public void warn(Object message, Throwable t) {
            }
        };

        ThreadPoolTaskExecutor taskExec = new BatchThreadPoolTaskExecutor();
        taskExec.setCorePoolSize(1);
        taskExec.setMaxPoolSize(1);
        taskExec.initialize();

        AsyncBatchExecutor.logOutputTaskExecutor(log, taskExec);
    }

    /**
     * testCheckEndFile001
     * @throws Exception
     */
    public void testCheckEndFile001() throws Exception {
        String endFilePath = null;
        boolean result = AsyncBatchExecutor.checkEndFile(endFilePath);
        assertFalse(result);
    }

    /**
     * testCheckEndFile002
     * @throws Exception
     */
    public void testCheckEndFile002() throws Exception {
        // eXgf[^ݒ
        URL srcUrl = this.getClass().getResource("/testdata/test01.txt");

        String endFilePath = srcUrl.getPath();
        boolean result = AsyncBatchExecutor.checkEndFile(endFilePath);
        assertTrue(result);
    }

    /**
     * testCheckTaskQueue001
     * @throws Exception
     */
    public void testCheckTaskQueue001() throws Exception {
        ThreadPoolTaskExecutor taskExec = new BatchThreadPoolTaskExecutor();
        taskExec.setCorePoolSize(1);
        taskExec.setMaxPoolSize(1);
        taskExec.initialize();

        boolean result = AsyncBatchExecutor.checkTaskQueue(taskExec);
        assertTrue(result);
    }

    /**
     * testCheckTaskQueue002
     * @throws Exception
     */
    public void testCheckTaskQueue002() throws Exception {
        ThreadPoolTaskExecutor taskExec = new ThreadPoolTaskExecutor();
        taskExec.setCorePoolSize(1);
        taskExec.setMaxPoolSize(1);
        taskExec.setQueueCapacity(1);
        taskExec.initialize();

        Runnable task = new Runnable() {
            public void run() {
                try {
                    System.out.println("aaa");
                    Thread.sleep(5000);
                    System.out.println("bbb");
                } catch (InterruptedException e) {
                    System.out.println("ccc");
                }
            }
        };
        taskExec.execute(task);

        Thread.sleep(1000);

        boolean result = AsyncBatchExecutor.checkTaskQueue(taskExec);
        assertTrue(result);

        taskExec.shutdown();
        Thread.sleep(500);
    }

    /**
     * testCheckTaskQueue003
     * @throws Exception
     */
    public void testCheckTaskQueue003() throws Exception {
        ThreadPoolTaskExecutor taskExec = new BatchThreadPoolTaskExecutor();
        taskExec.setCorePoolSize(1);
        taskExec.setMaxPoolSize(1);
        taskExec.initialize();

        Runnable task = new Runnable() {
            public void run() {
                try {
                    System.out.println("aaa");
                    Thread.sleep(5000);
                    System.out.println("bbb");
                } catch (InterruptedException e) {
                    System.out.println("ccc");
                }
            }
        };
        taskExec.execute(task);

        Thread.sleep(1000);

        boolean result = AsyncBatchExecutor.checkTaskQueue(taskExec);
        assertFalse(result);

        taskExec.shutdown();
        Thread.sleep(500);
    }
}
